#!/usr/bin/python
# Coding: UTF-8

import unittest

import numpy as np
import scipy as sp

from PyEMD.compact import *


class CompactTest(unittest.TestCase):
    @staticmethod
    def create_signal():
        # Do NOT modify this function! If you do, the following can happen:
        # -- Possible test errors for filter (tolerace of np.allclose).
        # -- Error in the derivative test (analytical derivative is hardcoded)
        t = np.linspace(0.0, np.pi, 200)
        return 0.1 * np.cos(2.0 * np.pi * t)

    def test_TDMA(self):

        diags = np.array([0.5 * np.ones(10), 1.0 * np.ones(10), 0.5 * np.ones(10)])
        positions = [-1, 0, 1]
        tridiag = sp.sparse.spdiags(diags, positions, 10, 10).todense()

        # change some diagonal values to make sure it is working
        diags[0][3] = 2.0
        diags[1][1] = 2.0
        tridiag[3, 2] = 2.0
        tridiag[1, 1] = 2.0

        rhs = np.arange(10)

        answer = np.linalg.solve(tridiag, rhs)
        # result = TDMAsolver(*diags, rhs)
        result = TDMAsolver(diags[0], diags[1], diags[2], rhs)

        self.assertTrue(np.allclose(answer, result))

    def test_filter_off(self):
        S = self.create_signal()
        self.assertTrue(np.allclose(S, filt6(S, 0.5), atol=1e-5))

    def test_filter_small(self):
        S = self.create_signal()
        self.assertTrue(np.allclose(S, filt6(S, 0.45), atol=1e-5))

    def test_filter_medium(self):
        S = self.create_signal()
        self.assertTrue(np.allclose(S, filt6(S, 0.0), atol=1e-5))

    def test_filter_high(self):
        S = self.create_signal()
        self.assertTrue(np.allclose(S, filt6(S, -0.5), atol=1e-5))

    def test_pade6(self):
        t = np.linspace(0.0, np.pi, 200)
        S = self.create_signal()
        Sprime = -0.1 * 2.0 * np.pi * np.sin(2.0 * np.pi * t)
        self.assertTrue(np.allclose(Sprime, pade6(S, t[1] - t[0]), atol=1e-5))


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np

from PyEMD import EEMD


class EEMDTest(unittest.TestCase):
    @staticmethod
    def cmp_msg(a, b):
        return "Expected {}, Returned {}".format(a, b)

    @staticmethod
    def test_default_call_EEMD():
        T = np.arange(50)
        S = np.cos(T * 0.1)
        max_imf = 2

        eemd = EEMD()
        eemd(S, T, max_imf)

    def test_eemd_simpleRun(self):
        T = np.linspace(0, 1, 100)
        S = np.sin(2 * np.pi * T)

        config = {"processes": 1}
        eemd = EEMD(trials=10, max_imf=1, **config)
        eemd.EMD.FIXE_H = 5
        eemd.eemd(S)

        self.assertTrue("processes" in eemd.__dict__)
        self.assertTrue(eemd.processes == 1)

    def test_eemd_passingArgumentsViaDict(self):
        trials = 10
        noise_kind = "uniform"
        spline_kind = "linear"

        # Making sure that we are not testing default options
        eemd = EEMD()

        self.assertFalse(eemd.trials == trials, self.cmp_msg(eemd.trials, trials))

        self.assertFalse(eemd.noise_kind == noise_kind, self.cmp_msg(eemd.noise_kind, noise_kind))

        self.assertFalse(eemd.EMD.spline_kind == spline_kind, self.cmp_msg(eemd.EMD.spline_kind, spline_kind))

        # Testing for passing attributes via params
        params = {"trials": trials, "noise_kind": noise_kind, "spline_kind": spline_kind}
        eemd = EEMD(**params)

        self.assertTrue(eemd.trials == trials, self.cmp_msg(eemd.trials, trials))

        self.assertTrue(eemd.noise_kind == noise_kind, self.cmp_msg(eemd.noise_kind, noise_kind))

        self.assertTrue(eemd.EMD.spline_kind == spline_kind, self.cmp_msg(eemd.EMD.spline_kind, spline_kind))

    def test_eemd_passingArgumentsDirectly(self):
        trials = 10
        noise_kind = "uniform"
        spline_kind = "linear"

        # Making sure that we are not testing default options
        eemd = EEMD()

        self.assertFalse(eemd.trials == trials, self.cmp_msg(eemd.trials, trials))

        self.assertFalse(eemd.noise_kind == noise_kind, self.cmp_msg(eemd.noise_kind, noise_kind))

        self.assertFalse(eemd.EMD.spline_kind == spline_kind, self.cmp_msg(eemd.EMD.spline_kind, spline_kind))

        # Testing for passing attributes via params
        eemd = EEMD(trials=trials, noise_kind=noise_kind, spline_kind=spline_kind)

        self.assertTrue(eemd.trials == trials, self.cmp_msg(eemd.trials, trials))

        self.assertTrue(eemd.noise_kind == noise_kind, self.cmp_msg(eemd.noise_kind, noise_kind))

        self.assertTrue(eemd.EMD.spline_kind == spline_kind, self.cmp_msg(eemd.EMD.spline_kind, spline_kind))

    def test_eemd_unsupportedNoiseKind(self):
        noise_kind = "whoever_supports_this_is_wrong"
        eemd = EEMD(noise_kind=noise_kind)

        with self.assertRaises(ValueError):
            eemd.generate_noise(1.0, 100)

    def test_eemd_passingCustomEMD(self):

        spline_kind = "linear"
        params = {"spline_kind": spline_kind}

        eemd = EEMD()
        self.assertFalse(eemd.EMD.spline_kind == spline_kind, "Not" + self.cmp_msg(eemd.EMD.spline_kind, spline_kind))

        from PyEMD import EMD

        emd = EMD(**params)

        eemd = EEMD(ext_EMD=emd)
        self.assertTrue(eemd.EMD.spline_kind == spline_kind, self.cmp_msg(eemd.EMD.spline_kind, spline_kind))

    def test_eemd_noiseSeed(self):
        T = np.linspace(0, 1, 100)
        S = np.sin(2 * np.pi * T + 4 ** T) + np.cos((T - 0.4) ** 2)

        # Compare up to machine epsilon
        def cmpMachEps(x, y):
            return np.abs(x - y) <= 2 * np.finfo(x.dtype).eps

        config = {"processes": 1}
        eemd = EEMD(trials=10, **config)

        # First run random seed
        eIMF1 = eemd(S)

        # Second run with defined seed, diff than first
        eemd.noise_seed(12345)
        eIMF2 = eemd(S)

        # Extremly unlikely to have same seed, thus different results
        msg_false = "Different seeds, expected different outcomes"
        if eIMF1.shape == eIMF2.shape:
            self.assertFalse(np.all(cmpMachEps(eIMF1, eIMF2)), msg_false)

        # Third run with same seed as with 2nd
        eemd.noise_seed(12345)
        eIMF3 = eemd(S)

        # Using same seeds, thus expecting same results
        msg_true = "Used same seed, expected same results"
        self.assertTrue(np.all(cmpMachEps(eIMF2, eIMF3)), msg_true)

    def test_eemd_notParallel(self):
        S = np.random.random(100)

        eemd = EEMD(trials=5, max_imf=2, parallel=False)
        eemd.EMD.FIXE_H = 2
        eIMFs = eemd.eemd(S)

        self.assertTrue(eIMFs.shape[0] > 0)
        self.assertTrue(eIMFs.shape[1], len(S))
        self.assertFalse("pool" in eemd.__dict__)

    def test_imfs_and_residue_accessor(self):
        S = np.random.random(100)
        eemd = EEMD(trials=5, max_imf=2, parallel=False)
        eIMFs = eemd(S)

        imfs, residue = eemd.get_imfs_and_residue()
        self.assertEqual(eIMFs.shape[0], imfs.shape[0], "Compare number of components")
        self.assertEqual(len(residue), 100, "Check if residue exists")

    def test_imfs_and_residue_accessor2(self):
        eemd = EEMD()
        with self.assertRaises(ValueError):
            imfs, residue = eemd.get_imfs_and_residue()

    def test_separate_trends(self):
        T = np.linspace(0, 2 * np.pi, 100)
        S = np.sin(T) + 3 * np.sin(3 * T + 0.1) + 0.2 * (T + 0.5) * (T - 2)
        eemd = EEMD(trials=20, separate_trends=True)

        eIMFs = eemd(S)
        for imf in eIMFs[:-1]:
            self.assertLess(abs(imf.mean()), 0.5)
        self.assertGreaterEqual(eIMFs[-1].mean(), 1)

    def test_eemd_ensemble_stats(self):
        T = np.linspace(0, 2 * np.pi, 100)
        S = np.sin(T) + 3 * np.sin(3 * T + 0.1) + 0.2 * (T + 0.5) * (T - 2)
        eemd = EEMD(trials=20, separate_trends=True)

        eIMFs = eemd(S)
        self.assertEqual(type(eemd.all_imfs), dict, "All imfs are stored as a dict")
        self.assertTrue(np.all(eIMFs == eemd.ensemble_mean()), "eIMFs are the mean over ensemble")
        self.assertEqual(eemd.ensemble_count(), [len(imfs) for imfs in eemd.all_imfs.values()])
        self.assertEqual(type(eemd.ensemble_std()), np.ndarray, "Ensemble std exists and it's a numpy array")


if __name__ == "__main__":
    unittest.main()



import numpy as np
import pylab as plt
from scipy.signal import hilbert

from PyEMD import EMD


def instant_phase(imfs):
    """Extract analytical signal through Hilbert Transform."""
    analytic_signal = hilbert(imfs)  # Apply Hilbert transform to each row
    phase = np.unwrap(np.angle(analytic_signal))  # Compute angle between img and real
    return phase


# Define signal
t = np.linspace(0, 1, 200)
dt = t[1] - t[0]

sin = lambda x, p: np.sin(2 * np.pi * x * t + p)
S = 3 * sin(18, 0.2) * (t - 0.2) ** 2
S += 5 * sin(11, 2.7)
S += 3 * sin(14, 1.6)
S += 1 * np.sin(4 * 2 * np.pi * (t - 0.8) ** 2)
S += t ** 2.1 - t

# Compute IMFs with EMD
emd = EMD()
imfs = emd(S, t)

# Extract instantaneous phases and frequencies using Hilbert transform
instant_phases = instant_phase(imfs)
instant_freqs = np.diff(instant_phases) / (2 * np.pi * dt)

# Create a figure consisting of 3 panels which from the top are the input signal, IMFs and instantaneous frequencies
fig, axes = plt.subplots(3, figsize=(12, 12))

# The top panel shows the input signal
ax = axes[0]
ax.plot(t, S)
ax.set_ylabel("Amplitude [arb. u.]")
ax.set_title("Input signal")

# The middle panel shows all IMFs
ax = axes[1]
for num, imf in enumerate(imfs):
    ax.plot(t, imf, label="IMF %s" % (num + 1))

# Label the figure
ax.legend()
ax.set_ylabel("Amplitude [arb. u.]")
ax.set_title("IMFs")

# The bottom panel shows all instantaneous frequencies
ax = axes[2]
for num, instant_freq in enumerate(instant_freqs):
    ax.plot(t[:-1], instant_freq, label="IMF %s" % (num + 1))

# Label the figure
ax.legend()
ax.set_xlabel("Time [s]")
ax.set_ylabel("Inst. Freq. [Hz]")
ax.set_title("Huang-Hilbert Transform")

plt.tight_layout()
plt.savefig("hht_example", dpi=120)
plt.show()



import unittest

import numpy as np

from PyEMD import EMD


class EMDTest(unittest.TestCase):
    @staticmethod
    def test_default_call_EMD():
        T = np.arange(50)
        S = np.cos(T * 0.1)
        max_imf = 2

        emd = EMD()
        emd(S, T, max_imf)

    def test_different_length_input(self):
        T = np.arange(20)
        S = np.random.random(len(T) + 7)

        emd = EMD()
        with self.assertRaises(ValueError):
            emd.emd(S, T)

    def test_trend(self):
        """
        Input is trend. Expeting no shifting process.
        """
        emd = EMD()

        t = np.arange(0, 1, 0.01)
        S = 2 * t

        # Input - linear function f(t) = 2*t
        imfs = emd.emd(S, t)
        self.assertEqual(imfs.shape[0], 1, "Expecting single IMF")
        self.assertTrue(np.allclose(S, imfs[0]))

    def test_single_imf(self):
        """
        Input is IMF. Expecint single shifting.
        """

        def max_diff(a, b):
            return np.max(np.abs(a - b))

        emd = EMD()
        emd.FIXE_H = 2

        t = np.arange(0, 1, 0.001)
        c1 = np.cos(4 * 2 * np.pi * t)  # 2 Hz
        S = c1.copy()

        # Input - linear function f(t) = sin(2Hz t)
        imfs = emd.emd(S, t)
        self.assertEqual(imfs.shape[0], 1, "Expecting sin + trend")

        diff = np.allclose(imfs[0], c1)
        self.assertTrue(diff, "Expecting 1st IMF to be sin\nMaxDiff = " + str(max_diff(imfs[0], c1)))

        # Input - linear function f(t) = siin(2Hz t) + 2*t
        c2 = 5 * (t + 2)
        S += c2.copy()
        imfs = emd.emd(S, t)

        self.assertEqual(imfs.shape[0], 2, "Expecting sin + trend")
        diff1 = np.allclose(imfs[0], c1, atol=0.2)
        self.assertTrue(diff1, "Expecting 1st IMF to be sin\nMaxDiff = " + str(max_diff(imfs[0], c1)))
        diff2 = np.allclose(imfs[1], c2, atol=0.2)
        self.assertTrue(diff2, "Expecting 2nd IMF to be trend\nMaxDiff = " + str(max_diff(imfs[1], c2)))

    def test_emd_passArgsViaDict(self):
        FIXE = 10
        params = {"FIXE": FIXE, "nothing": 0}

        # First test without initiation
        emd = EMD()
        self.assertFalse(emd.FIXE == FIXE, "{} == {}".format(emd.FIXE, FIXE))

        # Second: test with passing
        emd = EMD(**params)
        self.assertTrue(emd.FIXE == FIXE, "{} == {}".format(emd.FIXE, FIXE))

    def test_emd_passImplicitParamsDirectly(self):
        FIXE = 10
        svar_thr = 0.2

        # First test without initiation
        emd = EMD()
        self.assertFalse(emd.FIXE == FIXE, "{} == {}".format(emd.FIXE, FIXE))

        # Second: test with passing
        emd = EMD(FIXE=FIXE, svar_thr=svar_thr, nothing=0)
        self.assertTrue(emd.FIXE == FIXE, "{} == {}".format(emd.FIXE, FIXE))
        self.assertTrue(emd.svar_thr == svar_thr, "{} == {}".format(emd.svar_thr, svar_thr))

    def test_emd_FIXE(self):
        T = np.linspace(0, 1, 100)
        c = np.sin(9 * 2 * np.pi * T)
        offset = 4
        S = c + offset

        emd = EMD()

        # Default state: converge
        self.assertTrue(emd.FIXE == 0)
        self.assertTrue(emd.FIXE_H == 0)

        # Set 1 iteration per each sift,
        # same as removing offset
        FIXE = 1
        emd.FIXE = FIXE

        # Check flags correctness
        self.assertTrue(emd.FIXE == FIXE)
        self.assertTrue(emd.FIXE_H == 0)

        # Extract IMFs
        IMFs = emd.emd(S)

        # Check that IMFs are correct
        self.assertTrue(np.allclose(IMFs[0], c))
        self.assertTrue(np.allclose(IMFs[1], offset))

    def test_emd_FIXEH(self):
        T = np.linspace(0, 2, 200)
        c1 = 1 * np.sin(11 * 2 * np.pi * T + 0.1)
        c2 = 11 * np.sin(1 * 2 * np.pi * T + 0.1)
        offset = 9
        S = c1 + c2 + offset

        emd = EMD()

        # Default state: converge
        self.assertTrue(emd.FIXE == 0)
        self.assertTrue(emd.FIXE_H == 0)

        # Set 5 iterations per each protoIMF
        FIXE_H = 6
        emd.FIXE_H = FIXE_H

        # Check flags correctness
        self.assertTrue(emd.FIXE == 0)
        self.assertTrue(emd.FIXE_H == FIXE_H)

        # Extract IMFs
        imfs = emd.emd(S)

        # Check that IMFs are correct
        self.assertTrue(imfs.shape[0] == 3)

        close_imf1 = np.allclose(c1[2:-2], imfs[0, 2:-2], atol=0.2)
        self.assertTrue(close_imf1)
        self.assertTrue(np.allclose(c1, imfs[0], atol=1.0))

        close_imf2 = np.allclose(c2[2:-2], imfs[1, 2:-2], atol=0.21)
        self.assertTrue(close_imf2)
        self.assertTrue(np.allclose(c2, imfs[1], atol=1.0))

        close_offset = np.allclose(offset, imfs[2, 2:-2], atol=0.1)
        self.assertTrue(close_offset)

        close_offset = np.allclose(offset, imfs[2, 1:-1], atol=0.5)
        self.assertTrue(close_offset)

    def test_emd_default(self):
        T = np.linspace(0, 2, 200)
        c1 = 1 * np.sin(11 * 2 * np.pi * T + 0.1)
        c2 = 11 * np.sin(1 * 2 * np.pi * T + 0.1)
        offset = 9
        S = c1 + c2 + offset

        emd = EMD(spline_kind="akima")
        imfs = emd.emd(S, T)
        self.assertTrue(imfs.shape[0] == 3)

        close_imfs1 = np.allclose(c1[2:-2], imfs[0, 2:-2], atol=0.21)
        self.assertTrue(close_imfs1)

        close_imfs2 = np.allclose(c2[2:-2], imfs[1, 2:-2], atol=0.24)
        self.assertTrue(close_imfs2)

        close_offset = np.allclose(offset, imfs[2, 1:-1], atol=0.5)
        self.assertTrue(close_offset)

    def test_max_iteration_flag(self):
        S = np.random.random(200)
        emd = EMD()
        emd.MAX_ITERATION = 10
        emd.FIXE = 20

        imfs = emd.emd(S)

        # There's not much to test, except that it doesn't fail.
        # With low MAX_ITERATION value for random signal it's
        # guaranteed to have at least 2 imfs.
        self.assertTrue(imfs.shape[0] > 1)

    def test_get_imfs_and_residue(self):
        S = np.random.random(200)
        emd = EMD(**{"MAX_ITERATION": 10, "FIXE": 20})
        all_imfs = emd(S, max_imf=3)

        imfs, residue = emd.get_imfs_and_residue()
        self.assertEqual(all_imfs.shape[0], imfs.shape[0] + 1, "Compare number of components")
        self.assertTrue(np.array_equal(all_imfs[:-1], imfs), "Shouldn't matter where imfs are from")
        self.assertTrue(np.array_equal(all_imfs[-1], residue), "Residue, if any, is the last row")

    def test_get_imfs_and_residue_without_running(self):
        emd = EMD()
        with self.assertRaises(ValueError):
            _, _ = emd.get_imfs_and_residue()

    def test_get_imfs_and_trend(self):
        emd = EMD()
        T = np.linspace(0, 2 * np.pi, 100)
        expected_trend = 5 * T
        S = 2 * np.sin(4.1 * 6.28 * T) + 1.2 * np.cos(7.4 * 6.28 * T) + expected_trend

        all_imfs = emd(S)
        imfs, trend = emd.get_imfs_and_trend()

        onset_trend = trend - trend.mean()
        onset_expected_trend = expected_trend - expected_trend.mean()
        self.assertEqual(all_imfs.shape[0], imfs.shape[0] + 1, "Compare number of components")
        self.assertTrue(np.array_equal(all_imfs[:-1], imfs), "Shouldn't matter where imfs are from")
        self.assertTrue(
            np.allclose(onset_trend, onset_expected_trend, rtol=0.1, atol=0.5),
            "Extracted trend should be close to the actual trend",
        )


if __name__ == "__main__":
    unittest.main()



"""Tests for checks.py."""
import unittest

import numpy as np

from PyEMD.checks import energy, mean_period, significance_aposteriori, significance_apriori, whitenoise_check


class TestCase(unittest.TestCase):
    """Test cases."""

    def test_mean_period(self):
        """Test to check if mean period output is correct."""
        T = np.linspace(0, 2, 100)
        S = np.sin(2 * np.pi * T)
        res = mean_period(S)
        self.assertEqual(type(res), float, "Default data type is float")
        self.assertTrue(res > 0, "mean-period cannot be zero")

    def test_mean_period_zero_peaks(self):
        """Tect to check if mean period function can handle zero peaks."""
        T = np.linspace(0, 2, 100)
        res = mean_period(T)
        self.assertEqual(res, len(T), "mean-period is same as signal length in case of monotonic curve")

    def test_energy(self):
        """Test to check if energy of signal is being computed properly."""
        T = np.linspace(0, 2, 200)
        S = np.sin(2 * 2 * np.pi * T)
        res = energy(S)
        self.assertEqual(type(res), np.float64, "Default data type is float")

    def test_significance_apriori(self):
        """a priori significance test."""
        T = np.linspace(0, 2, 200)
        S = np.sin(2 * 2 * np.pi * T)
        energy_density = energy(S) / len(S)
        res = significance_apriori(energy_density, 2, len(S), 0.9)
        self.assertEqual(type(res), bool, "Default data type is bool")

    def test_significance_aposteriori(self):
        """a posteriori significance test."""
        T = np.linspace(0, 2, 200)
        S = np.sin(2 * 2 * np.pi * T)
        energy_density = energy(S) / len(S)
        res = significance_aposteriori(energy_density, 2, len(S), 0.9)
        self.assertEqual(type(res), bool, "Default data type is bool")

    def test_whitenoise_check_apriori(self):
        """a priori whitenoise_check."""
        T = [np.linspace(0, i, 200) for i in range(5, 0, -1)]
        S = np.array([list(np.sin(2 * 2 * np.pi * i)) for i in T])
        res = whitenoise_check(S, test_name="apriori")
        self.assertEqual(type(res), dict or None, "Default data type is dict")

    def test_whitenoise_check_apriori_alpha(self):
        """a priori whitenoise_check with custom alpha."""
        T = [np.linspace(0, i, 200) for i in range(5, 0, -1)]
        S = np.array([list(np.sin(2 * 2 * np.pi * i)) for i in T])
        res = whitenoise_check(S, test_name="apriori", alpha=0.99)
        self.assertEqual(type(res), dict or None, "Default data type is dict")

    def test_whitenoise_check_alpha(self):
        """a posteriori whitenoise check with custom alpha value."""
        T = [np.linspace(0, i, 200) for i in range(5, 0, -1)]
        S = np.array([list(np.sin(2 * 2 * np.pi * i)) for i in T])
        res = whitenoise_check(S, alpha=0.9)
        self.assertEqual(type(res), dict or None, "Default data type is dict")

    def test_whitenoise_check_rescaling_imf(self):
        """a posteriori whitenoise check with custom rescaling imf."""
        T = [np.linspace(0, i, 200) for i in range(5, 0, -1)]
        S = np.array([list(np.sin(2 * 2 * np.pi * i)) for i in T])
        res = whitenoise_check(S, rescaling_imf=2)
        self.assertEqual(type(res), dict or None, "Default data type is dict")

    def test_whitenoise_check_nan_values(self):
        """whitenoise check with nan in IMF."""
        S = np.array([np.full(100, np.NaN) for i in range(5, 0, -1)])
        res = whitenoise_check(S)
        self.assertEqual(res, None, "Input NaN returns None")

    def test_invalid_alpha(self):
        """Test if invalid alpha return AssertionError."""
        S = np.array([np.full(100, np.NaN) for i in range(5, 0, -1)])
        self.assertRaises(AssertionError, whitenoise_check, S, alpha=1)
        self.assertRaises(AssertionError, whitenoise_check, S, alpha=0)
        self.assertRaises(AssertionError, whitenoise_check, S, alpha=-10)
        self.assertRaises(AssertionError, whitenoise_check, S, alpha=2)
        self.assertRaises(AssertionError, whitenoise_check, S, alpha="0.5")

    def test_invalid_test_name(self):
        """Test if invalid test return AssertionError."""
        S = np.random.random((5, 100))
        self.assertRaises(AssertionError, whitenoise_check, S, test_name="apri")
        self.assertRaises(AssertionError, whitenoise_check, S, test_name="apost")
        self.assertRaises(AssertionError, whitenoise_check, S, test_name=None)

    def test_invalid_input_type(self):
        """Test if invalid input type return AssertionError."""
        S = [np.full(100, np.NaN) for i in range(5, 0, -1)]
        self.assertRaises(AssertionError, whitenoise_check, S)
        self.assertRaises(AssertionError, whitenoise_check, 1)
        self.assertRaises(AssertionError, whitenoise_check, 1.2)
        self.assertRaises(AssertionError, whitenoise_check, "[1,2,3,4,5]")

    def test_invalid_rescaling_imf(self):
        """Test if invalid rescaling imf return AssertionError."""
        T = [np.linspace(0, i, 200) for i in range(5, 0, -1)]
        S = np.array([list(np.sin(2 * 2 * np.pi * i)) for i in T])
        self.assertRaises(AssertionError, whitenoise_check, S, rescaling_imf=10)
        self.assertRaises(AssertionError, whitenoise_check, S, rescaling_imf=1.2)

    def test_empty_input_imf(self):
        """Test if empty IMF input return AssertionError."""
        T1 = np.array([[], []])
        T2 = np.array([])
        res1 = whitenoise_check(T1)
        res2 = whitenoise_check(T2)
        self.assertEqual(res1, None, "Empty input returns None")
        self.assertEqual(res2, None, "Empty input returns None")


if __name__ == "__main__":
    unittest.main()



import logging

__version__ = "1.2.2"
logger = logging.getLogger("pyemd")

from PyEMD.CEEMDAN import CEEMDAN  # noqa
from PyEMD.EEMD import EEMD  # noqa
from PyEMD.EMD import EMD  # noqa
from PyEMD.visualisation import Visualisation  # noqa



#!/usr/bin/python
# Coding: UTF-8

import unittest

import numpy as np

from PyEMD.EMD2d import EMD2D


@unittest.skip("Not supported until supported")
class ImageEMDTest(unittest.TestCase):
    def setUp(self) -> None:
        self.emd2d = EMD2D()

    @staticmethod
    def _generate_image(r=64, c=64):
        return np.random.random((r, c))

    @staticmethod
    def _generate_linear_image(r=16, c=16):
        rows = np.arange(r)
        return np.repeat(rows, c).reshape(r, c)

    @staticmethod
    def _generate_Gauss(x, y, pos, std, amp=1):
        x_s = x - pos[0]
        y_s = y - pos[1]
        x2 = x_s * x_s
        y2 = y_s * y_s

        exp = np.exp(-(x2 + y2) / (2 * std * std))
        # exp[exp<1e-6] = 0
        scale = amp / np.linalg.norm(exp)

        return scale * exp

    def test_default_call_EMD2d(self):
        x = np.arange(50)
        y = np.arange(50)
        xv, yv = np.meshgrid(x, y)
        pos = (10, 20)
        std = 5
        img = self._generate_Gauss(xv, yv, pos, std)

        max_imf = 2

        emd2d = EMD2D()
        emd2d(img, max_imf)

    def test_endCondition_perfectReconstruction(self):
        c1 = self._generate_image()
        c2 = self._generate_image()
        IMFs = np.stack((c1, c2))
        org_img = np.sum(IMFs, axis=0)
        self.assertTrue(self.emd2d.end_condition(org_img, IMFs))

    def test_findExtrema_singleMax(self):
        x = np.arange(50)
        y = np.arange(50)
        xv, yv = np.meshgrid(x, y)
        pos = (10, 20)
        std = 5
        img_max = self._generate_Gauss(xv, yv, pos, std)

        idx_min, idx_max = self.emd2d.find_extrema(img_max)
        x_min, y_min = xv[idx_min], yv[idx_min]
        x_max, y_max = xv[idx_max], yv[idx_max]

        self.assertTrue((x_max, y_max) == pos)
        self.assertTrue(len(x_min) == 0)
        self.assertTrue(len(y_min) == 0)

    def test_findExtrema_singleMin(self):
        x = np.arange(50)
        y = np.arange(50)
        xv, yv = np.meshgrid(x, y)
        pos = (10, 20)
        std = 5
        img_max = (-1) * self._generate_Gauss(xv, yv, pos, std, 10)

        idx_min, idx_max = self.emd2d.find_extrema(img_max)
        x_min, y_min = xv[idx_min], yv[idx_min]
        x_max, y_max = xv[idx_max], yv[idx_max]

        self.assertTrue((x_min, y_min) == pos)
        self.assertTrue(len(x_max) == 0)
        self.assertTrue(len(y_max) == 0)

    def test_findExtrema_general(self):
        x = np.arange(50)
        y = np.arange(50)
        xv, yv = np.meshgrid(x, y)

        min_peaks = [((5, 40), 4, -1), ((34, 10), 2, -3)]
        max_peaks = [((10, 20), 5, 2), ((25, 25), 3, 1), ((40, 5), 3, 3)]

        # Construct image with few Gausses
        img = np.zeros(xv.shape)
        for peak in max_peaks + min_peaks:
            img = img + self._generate_Gauss(xv, yv, peak[0], peak[1], peak[2])

        # Extract extrema
        idx_min, idx_max = self.emd2d.find_extrema(img)
        x_min = xv[idx_min].tolist()
        y_min = yv[idx_min].tolist()
        x_max = xv[idx_max].tolist()
        y_max = yv[idx_max].tolist()

        # Confirm that all peaks found - number
        self.assertTrue(len(x_min) == len(min_peaks))
        self.assertTrue(len(y_min) == len(min_peaks))
        self.assertTrue(len(x_max) == len(max_peaks))
        self.assertTrue(len(y_max) == len(max_peaks))

        for peak in min_peaks:
            peak_pos = peak[0]
            x_min.remove(peak_pos[0])
            y_min.remove(peak_pos[1])

        for peak in max_peaks:
            peak_pos = peak[0]
            x_max.remove(peak_pos[0])
            y_max.remove(peak_pos[1])

        # Confirm that all peaks found - exact position
        self.assertTrue(len(x_min) == 0)
        self.assertTrue(len(y_min) == 0)
        self.assertTrue(len(x_max) == 0)
        self.assertTrue(len(y_max) == 0)

    def test_splinePoints_SBS_simpleGrid(self):
        # Test points - leave space inbetween for interpolation
        X = np.arange(5) * 2
        Y = np.arange(5) * 2
        xm, ym = np.meshgrid(X, Y)

        xmf = xm.flatten()
        ymf = ym.flatten()

        # Constant value image
        zf = np.ones(xmf.size)

        # Interpolation grid
        xi = np.arange(10)
        yi = np.arange(10)

        # interpolated_image == np.ones((5,5))
        interpolated_image = self.emd2d.spline_points(xmf, ymf, zf, xi, yi)

        # It is expected, that interpolation on constant will produce
        # constant image
        self.assertTrue(np.allclose(interpolated_image, 1))

    def test_splinePoints_SBS_linearGrid(self):
        X = np.arange(5) * 2
        Y = np.arange(5) * 2
        xm, ym = np.meshgrid(X, Y)

        xmf = xm.flatten()
        ymf = ym.flatten()

        # Linear value image
        z = np.repeat(np.arange(0, 10, 2)[None, :], 5, axis=0)
        zf = z.flatten()

        # Interpolation grid
        xi = np.arange(9)
        yi = np.arange(9)

        # interpolated_image[row] == row
        interpolated_image = self.emd2d.spline_points(xmf, ymf, zf, xi, yi)

        # Since interpolation is on linear function, each row should have
        # value equal to position, i.e. z[0]=(0...), ,,,, z[n] = (n...n).
        for n in range(xi.size):
            nth_row = interpolated_image[n]
            self.assertTrue(np.allclose(nth_row, n))

    def test_emd2d_noExtrema(self):
        linear_image = self._generate_linear_image()

        IMFs = self.emd2d.emd(linear_image)

        self.assertTrue(np.all(linear_image == IMFs))

    def test_emd2d_simpleIMF(self):
        rows, cols = 128, 128

        # Sinusoidal IMF
        X = np.arange(cols)[None, :].T
        Y = np.arange(rows)
        sin_1d = np.sin(Y * 0.3)
        cos_1d = np.cos(X * 0.4)
        comp_2d = 10 * cos_1d * sin_1d
        comp_2d -= np.mean(comp_2d)

        image = comp_2d
        IMFs = self.emd2d.emd(image)

        # Image = IMF + noise
        self.assertTrue(IMFs.shape[0] <= 2, "Depending on spline, there should be an IMF and possibly trend")

        self.assertTrue(
            np.allclose(IMFs[0], image, atol=0.5), "Output: \n" + str(IMFs[0]) + "\nInput: \n" + str(comp_2d)
        )

    def test_emd2d_linearBackground_simpleIMF(self):
        rows, cols = 128, 128
        linear_background = 0.1 * self._generate_linear_image(rows, cols)

        # Sinusoidal IMF
        X = np.arange(cols)[None, :].T
        Y = np.arange(rows)
        x_comp_1d = np.sin(X * 0.5)
        y_comp_1d = np.sin(Y * 0.2)
        comp_2d = 5 * x_comp_1d * y_comp_1d

        image = linear_background + comp_2d
        IMFs = self.emd2d.emd(image)

        # Check that only two IMFs were extracted
        self.assertTrue(IMFs.shape == (2, rows, cols), "Shape is " + str(IMFs.shape))

        # First IMF should be sin
        self.assertTrue(
            np.allclose(IMFs[0], comp_2d, atol=1.0), "Output: \n" + str(IMFs[0]) + "\nInput: \n" + str(comp_2d)
        )

        # Second IMF should be linear trend
        self.assertTrue(np.allclose(IMFs[1], linear_background, atol=1.0))

    def test_emd2d_linearBackground_simpleIMF_FIXE(self):
        rows, cols = 128, 128
        linear_background = 0.1 * self._generate_linear_image(rows, cols)

        # Sinusoidal IMF
        X = np.arange(cols)[None, :].T
        Y = np.arange(rows)
        x_comp_1d = np.sin(X * 0.5)
        y_comp_1d = np.sin(Y * 0.2)
        comp_2d = 5 * x_comp_1d * y_comp_1d

        image = linear_background + comp_2d
        emd2d = EMD2D()
        emd2d.FIXE = 10
        IMFs = emd2d.emd(image)

        # Check that only two IMFs were extracted
        self.assertTrue(IMFs.shape == (2, rows, cols), "Shape is " + str(IMFs.shape))

        # First IMF should be sin
        self.assertTrue(
            np.allclose(IMFs[0], comp_2d, atol=1.0), "Output: \n" + str(IMFs[0]) + "\nInput: \n" + str(comp_2d)
        )

        # Second IMF should be linear trend
        self.assertTrue(np.allclose(IMFs[1], linear_background, atol=1.0))

    def test_emd2d_linearBackground_simpleIMF_FIXE_H(self):
        rows, cols = 128, 128
        linear_background = 0.1 * self._generate_linear_image(rows, cols)

        # Sinusoidal IMF
        X = np.arange(cols)[None, :].T
        Y = np.arange(rows)
        x_comp_1d = np.sin(X * 0.5)
        y_comp_1d = np.sin(Y * 0.2)
        comp_2d = 5 * x_comp_1d * y_comp_1d

        image = linear_background + comp_2d
        emd2D = EMD2D()
        emd2D.FIXE_H = 10
        IMFs = emd2D.emd(image)

        # Check that only two IMFs were extracted
        self.assertTrue(IMFs.shape == (2, rows, cols), "Shape is " + str(IMFs.shape))

        # First IMF should be sin
        self.assertTrue(
            np.allclose(IMFs[0], comp_2d, atol=1.0), "Output: \n" + str(IMFs[0]) + "\nInput: \n" + str(comp_2d)
        )

        # Second IMF should be linear trend
        self.assertTrue(np.allclose(IMFs[1], linear_background, atol=1.0))

    def test_emd2d_passArgsViaDict(self):

        FIXE = 10
        params = {"FIXE": FIXE}
        emd2D = EMD2D(**params)

        self.assertTrue(emd2D.FIXE == FIXE, "Received {}, Expceted {}".format(emd2D.FIXE, FIXE))

    def test_emd2d_limitImfNo(self):

        # Create image
        rows, cols = 128, 128
        linear_background = 0.2 * self._generate_linear_image(rows, cols)

        # Sinusoidal IMF
        X = np.arange(cols)[None, :].T
        Y = np.arange(rows)
        x_comp_1d = np.sin(X * 0.3) + np.cos(X * 2.9) ** 2
        y_comp_1d = np.sin(Y * 0.2)
        comp_2d = 10 * x_comp_1d * y_comp_1d
        comp_2d = comp_2d

        image = linear_background + comp_2d

        # Limit number of IMFs
        max_imf = 2

        # decompose image
        IMFs = self.emd2d.emd(image, max_imf=max_imf)

        # It should have no more than 2 (max_imf)
        self.assertTrue(IMFs.shape[0] == max_imf)


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np

from PyEMD import EMD
from PyEMD.visualisation import Visualisation


class VisTest(unittest.TestCase):
    def test_instantiation(self):
        emd = EMD()
        with self.assertRaises(ValueError):
            Visualisation(emd)

    def test_instantiation2(self):
        t = np.linspace(0, 1, 50)
        S = t + np.cos(np.cos(4.0 * t ** 2))
        emd = EMD()
        emd.emd(S, t)
        imfs, res = emd.get_imfs_and_residue()
        vis = Visualisation(emd)
        self.assertTrue(np.alltrue(vis.imfs == imfs))
        self.assertTrue(np.alltrue(vis.residue == res))

    def test_check_imfs(self):
        vis = Visualisation()
        imfs = np.arange(50).reshape(2, 25)
        res = np.arange(25)
        imfs, res = vis._check_imfs(imfs, res, False)
        self.assertEqual(len(imfs), 2)

    def test_check_imfs2(self):
        vis = Visualisation()
        with self.assertRaises(AttributeError):
            vis._check_imfs(None, None, False)

    def test_check_imfs3(self):
        vis = Visualisation()
        imfs = np.arange(50).reshape(2, 25)

        out_imfs, out_res = vis._check_imfs(imfs, None, False)

        self.assertTrue(np.alltrue(imfs == out_imfs))
        self.assertIsNone(out_res)

    def test_check_imfs4(self):
        vis = Visualisation()
        imfs = np.arange(50).reshape(2, 25)
        with self.assertRaises(AttributeError):
            vis._check_imfs(imfs, None, True)

    def test_check_imfs5(self):
        t = np.linspace(0, 1, 50)
        S = t + np.cos(np.cos(4.0 * t ** 2))
        emd = EMD()
        emd.emd(S, t)
        imfs, res = emd.get_imfs_and_residue()
        vis = Visualisation(emd)
        imfs2, res2 = vis._check_imfs(imfs, res, False)
        self.assertTrue(np.alltrue(imfs == imfs2))
        self.assertTrue(np.alltrue(res == res2))

    def test_plot_imfs(self):
        vis = Visualisation()
        with self.assertRaises(AttributeError):
            vis.plot_imfs()

    # Does not work for Python 2.7 (TravisCI), even with Agg backend
    # def test_plot_imfs2(self):
    #     t = np.linspace(0, 1, 50)
    #     S = t + np.cos(np.cos(4.*t**2))
    #     emd = EMD()
    #     emd.emd(S, t)
    #     vis = Visualisation(emd)
    #     vis.plot_imfs()

    def test_calc_instant_phase(self):
        sig = np.arange(10)
        vis = Visualisation()
        phase = vis._calc_inst_phase(sig, None)
        assert len(sig) == len(phase)

    def test_calc_instant_phase2(self):
        t = np.linspace(0, 1, 50)
        S = t + np.cos(np.cos(4.0 * t ** 2))
        emd = EMD()
        imfs = emd.emd(S, t)
        vis = Visualisation()
        phase = vis._calc_inst_phase(imfs, 0.4)
        assert len(imfs) == len(phase)

    def test_calc_instant_phase3(self):
        t = np.linspace(0, 1, 50)
        S = t + np.cos(np.cos(4.0 * t ** 2))
        emd = EMD()
        imfs = emd.emd(S, t)
        vis = Visualisation()
        with self.assertRaises(AssertionError):
            _ = vis._calc_inst_phase(imfs, 0.8)

    def test_calc_instant_freq_alphaNone(self):
        t = np.linspace(0, 1, 50)
        S = t + np.cos(np.cos(4.0 * t ** 2))
        emd = EMD()
        imfs = emd.emd(S, t)
        vis = Visualisation()
        freqs = vis._calc_inst_freq(imfs, t, False, None)
        self.assertEqual(imfs.shape, freqs.shape)

    def test_calc_instant_freq(self):
        t = np.linspace(0, 1, 50)
        S = t + np.cos(np.cos(4.0 * t ** 2))
        emd = EMD()
        imfs = emd.emd(S, t)
        vis = Visualisation()
        freqs = vis._calc_inst_freq(imfs, t, False, 0.4)
        self.assertEqual(imfs.shape, freqs.shape)

    def test_plot_instant_freq(self):
        vis = Visualisation()
        t = np.arange(20)
        with self.assertRaises(AttributeError):
            vis.plot_instant_freq(t)



#!/usr/bin/python
# Coding: UTF-8
import unittest

import numpy as np

try:
    from PyEMD.BEMD import BEMD
except (ImportError, ModuleNotFoundError):
    # Not supported until supported.
    pass


@unittest.skip("Not supported until supported")
class BEMDTest(unittest.TestCase):
    def setUp(self) -> None:
        self.bemd = BEMD()

    @staticmethod
    def _generate_image(r=64, c=64):
        return np.random.random((r, c))

    @staticmethod
    def _generate_linear_image(r=16, c=16):
        rows = np.arange(r)
        return np.repeat(rows, c).reshape(r, c)

    @staticmethod
    def _generate_Gauss(x, y, pos, std, amp=1):
        x_s = x - pos[0]
        y_s = y - pos[1]
        x2 = x_s * x_s
        y2 = y_s * y_s
        exp = np.exp(-(x2 + y2) / (2 * std * std))
        # exp[exp<1e-6] = 0
        scale = amp / np.linalg.norm(exp)
        return scale * exp

    @classmethod
    def _sin(cls, x_n=128, y_n=128, x_f=[1], y_f=[0], dx=0, dy=0):
        x = np.linspace(0, 1, x_n) - dx
        y = np.linspace(0, 1, y_n) - dy
        xv, yv = np.meshgrid(x, y)
        img = np.zeros(xv.shape)
        for f in x_f:
            img += np.sin(f * 2 * np.pi * xv)
        for f in y_f:
            img += np.cos(f * 2 * np.pi * yv)
        return 255 * (img - img.min()) / (img.max() - img.min())

    def test_extract_maxima(self):
        image = self._sin(x_n=32, y_n=32, y_f=[1], dy=1)
        max_peak_x, max_peak_y = BEMD.extract_maxima_positions(image)
        self.assertEqual(max_peak_x.size, 6)  # Clustering
        self.assertEqual(max_peak_y.size, 6)  # Clustering

    def test_extract_minima(self):
        image = self._sin(x_n=64, y_n=64, y_f=[2])
        min_peak_x, min_peak_y = BEMD.extract_minima_positions(image)
        self.assertEqual(min_peak_x.size, 16)  # Clustering
        self.assertEqual(min_peak_y.size, 16)  # Clustering

    def test_find_extrema(self):
        image = self._sin()
        min_peaks, max_peaks = BEMD.find_extrema_positions(image)
        self.assertTrue(isinstance(min_peaks, tuple))
        self.assertTrue(isinstance(min_peaks[0], np.ndarray))
        self.assertTrue(isinstance(max_peaks[1], np.ndarray))

    def test_default_call_BEMD(self):
        x = np.arange(50)
        y = np.arange(50)
        xv, yv = np.meshgrid(x, y)
        img = self._generate_Gauss(xv, yv, (10, 20), 5)
        img += self._sin(x_n=x.size, y_n=y.size)

        max_imf = 2
        self.bemd(img, max_imf)

    def test_endCondition_perfectReconstruction(self):
        c1 = self._generate_image()
        c2 = self._generate_image()
        IMFs = np.stack((c1, c2))
        org_img = np.sum(IMFs, axis=0)
        self.assertTrue(self.bemd.end_condition(org_img, IMFs))

    def test_bemd_simpleIMF(self):
        image = self._sin(x_f=[3, 5], y_f=[2])
        IMFs = self.bemd(image)

        # One of the reasons this algorithm isn't the preferred one
        self.assertTrue(IMFs.shape[0] == 7, "Depending on spline, there should be an IMF and possibly trend")

    def test_bemd_limitImfNo(self):

        # Create image
        rows, cols = 64, 64
        linear_background = 0.2 * self._generate_linear_image(rows, cols)

        # Sinusoidal IMF
        X = np.arange(cols)[None, :].T
        Y = np.arange(rows)
        x_comp_1d = np.sin(X * 0.3) + np.cos(X * 2.9) ** 2
        y_comp_1d = np.sin(Y * 0.2)
        comp_2d = 10 * x_comp_1d * y_comp_1d
        comp_2d = comp_2d

        image = linear_background + comp_2d

        # Limit number of IMFs
        max_imf = 2

        # decompose image
        IMFs = self.bemd(image, max_imf=max_imf)

        # It should have no more than 2 (max_imf) + residue
        self.assertEqual(IMFs.shape[0], 1 + max_imf)


if __name__ == "__main__":
    unittest.main()



# Author: Dawid Laszuk
# Last update: 7/07/2017
import numpy as np
import pylab as plt
from PyEMD import EMD2D

# Generate image
print("Generating image... ", end="")
rows, cols = 1024, 1024
row_scale, col_scale = 256, 256
x = np.arange(rows) / float(row_scale)
y = np.arange(cols).reshape((-1, 1)) / float(col_scale)

pi2 = 2 * np.pi
img = np.zeros((rows, cols))
img = img + np.sin(2 * pi2 * x) * np.cos(y * 4 * pi2 + 4 * x * pi2)
img = img + 3 * np.sin(2 * pi2 * x) + 2
img = img + 5 * x * y + 2 * (y - 0.2) * y
print("Done")

# Perform decomposition
print("Performing decomposition... ", end="")
emd2d = EMD2D()
# emd2d.FIXE_H = 5
IMFs = emd2d.emd(img, max_imf=4)
imfNo = IMFs.shape[0]
print("Done")

print("Plotting results... ", end="")

# Save image for preview
plt.figure(figsize=(4, 4 * (imfNo + 1)))
plt.subplot(imfNo + 1, 1, 1)
plt.imshow(img)
plt.colorbar()
plt.title("Input image")

# Save reconstruction
for n, imf in enumerate(IMFs):
    plt.subplot(imfNo + 1, 1, n + 2)
    plt.imshow(imf)
    plt.colorbar()
    plt.title("IMF %i" % (n + 1))

plt.savefig("image_decomp")
print("Done")



import unittest

import numpy as np

from PyEMD.utils import get_timeline


class MyTestCase(unittest.TestCase):
    def test_get_timeline_default_dtype(self):
        S = np.random.random(100)
        T = get_timeline(len(S))

        self.assertEqual(len(T), len(S), "Lengths must be equal")
        self.assertEqual(T.dtype, np.int64, "Default dtype is np.int64")
        self.assertEqual(T[-1], len(S) - 1, "Range is kept")

    def test_get_timeline_signal_dtype(self):
        S = np.random.random(100)
        T = get_timeline(len(S), dtype=S.dtype)

        self.assertEqual(len(T), len(S), "Lengths must be equal")
        self.assertEqual(T.dtype, S.dtype, "Dtypes must be equal")
        self.assertEqual(T[-1], len(S) - 1, "Range is kept")

    def test_get_timeline_does_not_overflow_int16(self):
        S = np.random.randint(100, size=(np.iinfo(np.int16).max + 10,), dtype=np.int16)
        T = get_timeline(len(S), dtype=S.dtype)

        self.assertGreater(len(S), np.iinfo(S.dtype).max, "Length of the signal is greater than its type max value")
        self.assertEqual(len(T), len(S), "Lengths must be equal")
        self.assertEqual(T[-1], len(S) - 1, "Range is kept")
        self.assertEqual(T.dtype, np.uint16, "UInt16 is the min type that matches requirements")

    def test_get_timeline_does_not_overflow_float16(self):
        S = np.random.random(int(np.finfo(np.float16).max) + 5).astype(dtype=np.float16)
        T = get_timeline(len(S), dtype=S.dtype)

        self.assertGreater(len(S), np.finfo(S.dtype).max, "Length of the signal is greater than its type max value")
        self.assertEqual(len(T), len(S), "Lengths must be equal")
        self.assertEqual(T[-1], len(S) - 1, "Range is kept")
        self.assertEqual(T.dtype, np.float32, "Float32 is the min type that matches requirements")


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np

from PyEMD import CEEMDAN


class CEEMDANTest(unittest.TestCase):
    @staticmethod
    def cmp_msg(a, b):
        return "Expected {}, Returned {}".format(a, b)

    @staticmethod
    def test_default_call_CEEMDAN():
        T = np.arange(50)
        S = np.cos(T * 0.1)
        max_imf = 2

        ceemdan = CEEMDAN(trials=5)
        ceemdan(S, T, max_imf)

    def test_ceemdan_simpleRun(self):
        T = np.linspace(0, 1, 100)
        S = np.sin(2 * np.pi * T)

        config = {"processes": 1}
        ceemdan = CEEMDAN(trials=10, max_imf=1, **config)
        ceemdan.EMD.FIXE_H = 5
        ceemdan.ceemdan(S)
        self.assertTrue("processes" in ceemdan.__dict__)

    def test_ceemdan_completeRun(self):
        S = np.random.random(200)

        ceemdan = CEEMDAN()
        cIMFs = ceemdan(S)

        self.assertTrue(cIMFs.shape[0] > 1)
        self.assertTrue(cIMFs.shape[1] == S.size)

    def test_ceemdan_passingArgumentsViaDict(self):
        trials = 10
        noise_kind = "uniform"
        spline_kind = "linear"

        # Making sure that we are not testing default options
        ceemdan = CEEMDAN()

        self.assertFalse(ceemdan.trials == trials, self.cmp_msg(ceemdan.trials, trials))

        self.assertFalse(
            ceemdan.noise_kind == noise_kind,
            self.cmp_msg(ceemdan.noise_kind, noise_kind),
        )

        self.assertFalse(
            ceemdan.EMD.spline_kind == spline_kind,
            self.cmp_msg(ceemdan.EMD.spline_kind, spline_kind),
        )

        # Testing for passing attributes via params
        params = {
            "trials": trials,
            "noise_kind": noise_kind,
            "spline_kind": spline_kind,
        }
        ceemdan = CEEMDAN(**params)

        self.assertTrue(ceemdan.trials == trials, self.cmp_msg(ceemdan.trials, trials))

        self.assertTrue(
            ceemdan.noise_kind == noise_kind,
            self.cmp_msg(ceemdan.noise_kind, noise_kind),
        )

        self.assertTrue(
            ceemdan.EMD.spline_kind == spline_kind,
            self.cmp_msg(ceemdan.EMD.spline_kind, spline_kind),
        )

    def test_ceemdan_testMaxImf(self):
        S = np.random.random(100)

        ceemdan = CEEMDAN(trials=10)

        max_imf = 1
        cIMFs = ceemdan(S, max_imf=max_imf)
        self.assertTrue(cIMFs.shape[0] == max_imf + 1)

        max_imf = 3
        cIMFs = ceemdan(S, max_imf=max_imf)
        self.assertTrue(cIMFs.shape[0] == max_imf + 1)

    @staticmethod
    def test_ceemdan_constantEpsilon():
        S = np.random.random(100)

        ceemdan = CEEMDAN(trials=10, max_imf=2)
        ceemdan.beta_progress = False
        ceemdan(S)

    @staticmethod
    def test_ceemdan_noiseKind_uniform():
        ceemdan = CEEMDAN()
        ceemdan.noise_kind = "uniform"
        ceemdan.generate_noise(1.0, 100)

    def test_ceemdan_noiseKind_unknown(self):
        ceemdan = CEEMDAN()
        ceemdan.noise_kind = "bernoulli"
        with self.assertRaises(ValueError):
            ceemdan.generate_noise(1.0, 100)

    def test_ceemdan_passingCustomEMD(self):

        spline_kind = "linear"
        params = {"spline_kind": spline_kind}

        ceemdan = CEEMDAN()
        self.assertFalse(
            ceemdan.EMD.spline_kind == spline_kind,
            "Not" + self.cmp_msg(ceemdan.EMD.spline_kind, spline_kind),
        )

        from PyEMD import EMD

        emd = EMD(**params)

        ceemdan = CEEMDAN(ext_EMD=emd)
        self.assertTrue(
            ceemdan.EMD.spline_kind == spline_kind,
            self.cmp_msg(ceemdan.EMD.spline_kind, spline_kind),
        )

    def test_ceemdan_noiseSeed(self):
        T = np.linspace(0, 1, 100)
        S = np.sin(2 * np.pi * T + 4 ** T) + np.cos((T - 0.4) ** 2)

        # Compare up to machine epsilon
        def cmpMachEps(x, y):
            return np.abs(x - y) <= 2 * np.finfo(x.dtype).eps

        ceemdan = CEEMDAN(trials=10)

        # First run random seed
        cIMF1 = ceemdan(S)

        # Second run with defined seed, diff than first
        ceemdan.noise_seed(12345)
        cIMF2 = ceemdan(S)

        # Extremly unlikely to have same seed, thus different results
        msg_false = "Different seeds, expected different outcomes"
        if cIMF1.shape == cIMF2.shape:
            self.assertFalse(np.all(cmpMachEps(cIMF1, cIMF2)), msg_false)

        # Third run with same seed as with 2nd
        ceemdan.noise_seed(12345)
        cIMF3 = ceemdan(S)

        # Using same seeds, thus expecting same results
        msg_true = "Used same seed, expected same results"
        self.assertTrue(np.all(cmpMachEps(cIMF2, cIMF3)), msg_true)

    def test_ceemdan_originalSignal(self):
        T = np.linspace(0, 1, 100)
        S = 2 * np.cos(3 * np.pi * T) + np.cos(2 * np.pi * T + 4 ** T)

        # Make a copy of S for comparsion
        Scopy = np.copy(S)

        # Compare up to machine epsilon
        def cmpMachEps(x, y):
            return np.abs(x - y) <= 2 * np.finfo(x.dtype).eps

        ceemdan = CEEMDAN(trials=10)
        ceemdan(S)

        # The original signal should not be changed after the 'ceemdan' function.
        msg_true = "Expected no change of the original signal"
        self.assertTrue(np.all(cmpMachEps(Scopy, S)), msg_true)

    def test_ceemdan_notParallel(self):
        S = np.random.random(100)

        ceemdan = CEEMDAN(parallel=False)
        cIMFs = ceemdan(S)

        self.assertTrue(cIMFs.shape[0] > 1)
        self.assertTrue(cIMFs.shape[1] == S.size)

    def test_imfs_and_residue_accessor(self):
        S = np.random.random(100)
        ceemdan = CEEMDAN(parallel=False)
        cIMFs = ceemdan(S)

        imfs, residue = ceemdan.get_imfs_and_residue()
        self.assertEqual(cIMFs.shape[0], imfs.shape[0], "Compare number of components")
        self.assertEqual(len(residue), 100, "Check if residue exists")

    def test_imfs_and_residue_accessor2(self):
        ceemdan = CEEMDAN()
        with self.assertRaises(ValueError):
            _, _ = ceemdan.get_imfs_and_residue()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# Coding: UTF-8

import unittest

import numpy as np

from PyEMD import EMD


class ExtremaTest(unittest.TestCase):
    def test_incorrectExtremaDetectionSetup(self):
        extrema_detection = "bubble_gum"

        # Sanity check
        emd = EMD()
        self.assertFalse(emd.extrema_detection == extrema_detection)

        # Assign incorrect extrema_detection
        emd.extrema_detection = extrema_detection
        self.assertTrue(emd.extrema_detection == extrema_detection)

        T = np.arange(10)
        S = np.sin(T)
        max_pos, max_val = np.random.random((2, 3))
        min_pos, min_val = np.random.random((2, 3))

        # Check for Exception
        with self.assertRaises(ValueError):
            emd.prepare_points(T, S, max_pos, max_val, min_pos, min_val)

    def test_wrong_extrema_detection_type(self):
        emd = EMD()
        emd.extrema_detection = "very_complicated"

        t = np.arange(10)
        s = np.array([-1, 0, 1, 0, -1, 0, 3, 0, -9, 0])

        with self.assertRaises(ValueError):
            emd.find_extrema(t, s)

    def test_find_extrema_simple(self):
        """Simple test for extrema."""
        emd = EMD()
        emd.extrema_detection = "simple"

        t = np.arange(10)
        s = np.array([-1, 0, 1, 0, -1, 0, 3, 0, -9, 0])
        expMaxPos = [2, 6]
        expMaxVal = [1, 3]
        expMinPos = [4, 8]
        expMinVal = [-1, -9]
        expZeros = t[s == 0]

        maxPos, maxVal, minPos, minVal, nz = emd.find_extrema(t, s)

        self.assertEqual(maxPos.tolist(), expMaxPos)
        self.assertEqual(maxVal.tolist(), expMaxVal)
        self.assertEqual(minPos.tolist(), expMinPos)
        self.assertEqual(minVal.tolist(), expMinVal)
        self.assertEqual(nz.tolist(), expZeros.tolist())

    def test_find_extrema_simple_repeat(self):
        r"""
        Test what happens in /^^\ situation, i.e.
        when extremum is somewhere between two consecutive pts.
        """
        emd = EMD()
        emd.extrema_detection = "simple"

        t = np.arange(2, 13)
        s = np.array([-1, 0, 1, 1, 0, -1, 0, 3, 0, -9, 0])
        expMaxPos = [4, 9]
        expMaxVal = [1, 3]
        expMinPos = [7, 11]
        expMinVal = [-1, -9]

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        self.assertEqual(maxPos.tolist(), expMaxPos)
        self.assertEqual(maxVal.tolist(), expMaxVal)
        self.assertEqual(minPos.tolist(), expMinPos)
        self.assertEqual(minVal.tolist(), expMinVal)

    def test_bound_extrapolation_simple(self):
        emd = EMD()
        emd.extrema_detection = "simple"
        emd.nbsym = 1
        emd.DTYPE = np.int64

        S = [0, -3, 1, 4, 3, 2, -2, 0, 1, 2, 1, 0, 1, 2, 5, 4, 0, -2, -1]
        S = np.array(S)
        T = np.arange(len(S))

        pp = emd.prepare_points

        # There are 4 cases for both (L)eft and (R)ight ends. In case of left (L) bound:
        # L1) ,/ -- ext[0] is min, s[0] < ext[1] (1st max)
        # L2) / -- ext[0] is min, s[0] > ext[1] (1st max)
        # L3) ^. -- ext[0] is max, s[0] > ext[1] (1st min)
        # L4) \ -- ext[0] is max, s[0] < ext[1] (1st min)

        # CASE 1
        # L1, R1 -- no edge MIN & no edge MIN
        s = S.copy()
        t = T.copy()

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        self.assertEqual([-1, 3, 9, 14, 20], maxExtrema[0].tolist())
        self.assertEqual([4, 4, 2, 5, 5], maxExtrema[1].tolist())
        self.assertEqual([-4, 1, 6, 11, 17, 23], minExtrema[0].tolist())
        self.assertEqual([-2, -3, -2, 0, -2, 0], minExtrema[1].tolist())

        # CASE 2
        # L2, R2 -- edge MIN, edge MIN
        s = S[1:-1].copy()
        t = np.arange(s.size)

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        self.assertEqual([-2, 2, 8, 13, 19], maxExtrema[0].tolist())
        self.assertEqual([4, 4, 2, 5, 5], maxExtrema[1].tolist())
        self.assertEqual([0, 5, 10, 16], minExtrema[0].tolist())
        self.assertEqual([-3, -2, 0, -2], minExtrema[1].tolist())

        # CASE 3
        # L3, R3 -- no edge MAX & no edge MAX
        s = S[2:-3].copy()
        t = np.arange(s.size)

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        self.assertEqual([-5, 1, 7, 12, 17], maxExtrema[0].tolist())
        self.assertEqual([2, 4, 2, 5, 2], maxExtrema[1].tolist())
        self.assertEqual([-2, 4, 9, 15], minExtrema[0].tolist())
        self.assertEqual([-2, -2, 0, 0], minExtrema[1].tolist())

        # CASE 4
        # L4, R4 -- edge MAX & edge MAX
        s = S[3:-4].copy()
        t = np.arange(s.size)

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        self.assertEqual([0, 6, 11], maxExtrema[0].tolist())
        self.assertEqual([4, 2, 5], maxExtrema[1].tolist())
        self.assertEqual([-3, 3, 8, 14], minExtrema[0].tolist())
        self.assertEqual([-2, -2, 0, 0], minExtrema[1].tolist())

    def test_find_extrema_parabol(self):
        """
        Simple test for extrema.
        """
        emd = EMD()
        emd.extrema_detection = "parabol"

        t = np.arange(10)
        s = np.array([-1, 0, 1, 0, -1, 0, 3, 0, -9, 0])
        expMaxPos = [2, 6]
        expMaxVal = [1, 3]
        expMinPos = [4, 8]
        expMinVal = [-1, -9]

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        self.assertEqual(maxPos.tolist(), expMaxPos)
        self.assertEqual(maxVal.tolist(), expMaxVal)
        self.assertEqual(minPos.tolist(), expMinPos)
        self.assertEqual(minVal.tolist(), expMinVal)

    def test_find_extrema_parabol_repeat(self):
        r"""
        Test what happens in /^^\ situation, i.e.
        when extremum is somewhere between two consecutive pts.
        """
        emd = EMD()
        emd.extrema_detection = "parabol"

        t = np.arange(2, 13)
        s = np.array([-1, 0, 1, 1, 0, -1, 0, 3, 0, -9, 0])
        expMaxPos = [4.5, 9]
        expMaxVal = [1.125, 3]
        expMinPos = [7, 11]
        expMinVal = [-1, -9]

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        self.assertEqual(maxPos.tolist(), expMaxPos)
        self.assertEqual(maxVal.tolist(), expMaxVal)
        self.assertEqual(minPos.tolist(), expMinPos)
        self.assertEqual(minVal.tolist(), expMinVal)

    def test_bound_extrapolation_parabol(self):
        emd = EMD()
        emd.extrema_detection = "parabol"
        emd.nbsym = 1
        emd.DTYPE = np.float64

        S = [0, -3, 1, 4, 3, 2, -2, 0, 1, 2, 1, 0, 1, 2, 5, 4, 0, -2, -1]
        S = np.array(S)
        T = np.arange(len(S))

        pp = emd.prepare_points

        # There are 4 cases for both (L)eft and (R)ight ends. In case of left (L) bound:
        # L1) ,/ -- ext[0] is min, s[0] < ext[1] (1st max)
        # L2) / -- ext[0] is min, s[0] > ext[1] (1st max)
        # L3) ^. -- ext[0] is max, s[0] > ext[1] (1st min)
        # L4) \ -- ext[0] is max, s[0] < ext[1] (1st min)

        # CASE 1
        # L1, R1 -- no edge MIN & no edge MIN
        s = S.copy()
        t = T.copy()

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        maxExtrema = np.round(maxExtrema, decimals=3)
        minExtrema = np.round(minExtrema, decimals=3)

        self.assertEqual([-1.393, 3.25, 9, 14.25, 20.083], maxExtrema[0].tolist())
        self.assertEqual([4.125, 4.125, 2, 5.125, 5.125], maxExtrema[1].tolist())
        self.assertEqual([-4.31, 0.929, 6.167, 11, 17.167, 23.333], minExtrema[0].tolist())
        self.assertEqual([-2.083, -3.018, -2.083, 0, -2.042, 0], minExtrema[1].tolist())

        # CASE 2
        # L2, R2 -- edge MIN, edge MIN
        s = S[1:-1].copy()
        t = T[1:-1].copy()

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        maxExtrema = np.round(maxExtrema, decimals=3)
        minExtrema = np.round(minExtrema, decimals=3)

        self.assertEqual([-1.25, 3.25, 9, 14.25, 19.75], maxExtrema[0].tolist())
        self.assertEqual([4.125, 4.125, 2, 5.125, 5.125], maxExtrema[1].tolist())
        self.assertEqual([1, 6.167, 11, 17], minExtrema[0].tolist())
        self.assertEqual([-3, -2.083, 0, -2], minExtrema[1].tolist())

        # CASE 3
        # L3, R3 -- no edge MAX & no edge MAX
        s = S[2:-3].copy()
        t = T[2:-3].copy()

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        maxExtrema = np.round(maxExtrema, decimals=3)
        minExtrema = np.round(minExtrema, decimals=3)

        self.assertEqual([-2.5, 3.25, 9, 14.25, 19.5], maxExtrema[0].tolist())
        self.assertEqual([2, 4.125, 2, 5.125, 2], maxExtrema[1].tolist())
        self.assertEqual([0.333, 6.167, 11, 17.5], minExtrema[0].tolist())
        self.assertEqual([-2.083, -2.083, 0, 0], minExtrema[1].tolist())

        # CASE 4
        # L4, R4 -- edge MAX & edge MAX
        s = S[3:-4].copy()
        t = T[3:-4].copy()

        maxPos, maxVal, minPos, minVal, _ = emd.find_extrema(t, s)

        # Should extrapolate left and right bounds
        maxExtrema, minExtrema = pp(t, s, maxPos, maxVal, minPos, minVal)

        maxExtrema = np.round(maxExtrema, decimals=3)
        minExtrema = np.round(minExtrema, decimals=3)

        self.assertEqual([3, 9, 14], maxExtrema[0].tolist())
        self.assertEqual([4, 2, 5], maxExtrema[1].tolist())
        self.assertEqual([-0.167, 6.167, 11, 17], minExtrema[0].tolist())
        self.assertEqual([-2.083, -2.083, 0, 0], minExtrema[1].tolist())

    # TODO:
    #   - nbsym > 1


if __name__ == "__main__":
    unittest.main()



# Author: Dawid Laszuk
# Last update: 7/07/2017
import numpy as np
import pylab as plt
from PyEMD import EMD

# Define signal
t = np.linspace(0, 1, 200)
s = np.cos(11 * 2 * np.pi * t * t) + 6 * t * t

# Execute EMD on signal
IMF = EMD().emd(s, t)
N = IMF.shape[0] + 1

# Plot results
plt.subplot(N, 1, 1)
plt.plot(t, s, "r")
plt.title("Input signal: $S(t)=cos(22\pi t^2) + 6t^2$")
plt.xlabel("Time [s]")

for n, imf in enumerate(IMF):
    plt.subplot(N, 1, n + 2)
    plt.plot(t, imf, "g")
    plt.title("IMF " + str(n + 1))
    plt.xlabel("Time [s]")

plt.tight_layout()
plt.savefig("simple_example")
plt.show()



# Author: Dawid Laszuk
# Last update: 7/07/2017
from PyEMD import EEMD
import numpy as np
import pylab as plt

# Define signal
t = np.linspace(0, 1, 200)

sin = lambda x, p: np.sin(2 * np.pi * x * t + p)
S = 3 * sin(18, 0.2) * (t - 0.2) ** 2
S += 5 * sin(11, 2.7)
S += 3 * sin(14, 1.6)
S += 1 * np.sin(4 * 2 * np.pi * (t - 0.8) ** 2)
S += t ** 2.1 - t

# Assign EEMD to `eemd` variable
eemd = EEMD()

# Say we want detect extrema using parabolic method
emd = eemd.EMD
emd.extrema_detection = "parabol"

# Execute EEMD on S
eIMFs = eemd.eemd(S, t)
nIMFs = eIMFs.shape[0]

# Plot results
plt.figure(figsize=(12, 9))
plt.subplot(nIMFs + 1, 1, 1)
plt.plot(t, S, "r")

for n in range(nIMFs):
    plt.subplot(nIMFs + 1, 1, n + 2)
    plt.plot(t, eIMFs[n], "g")
    plt.ylabel("eIMF %i" % (n + 1))
    plt.locator_params(axis="y", nbins=5)

plt.xlabel("Time [s]")
plt.tight_layout()
plt.savefig("eemd_example", dpi=120)
plt.show()



#!/usr/bin/python
# Coding: UTF-8

import unittest

import numpy as np

from PyEMD import EMD
from PyEMD.splines import *


class IMFTest(unittest.TestCase):
    """
    Since these tests depend heavily on NumPy & SciPy,
    make sure you have NumPy >= 1.12 and SciPy >= 0.19.
    """

    def test_unsupporter_spline(self):
        emd = EMD()
        emd.spline_kind = "waterfall"

        S = np.random.random(20)

        with self.assertRaises(ValueError):
            emd.emd(S)

    def test_akima(self):
        dtype = np.float32

        emd = EMD()
        emd.spline_kind = "akima"
        emd.DTYPE = dtype

        # Test error: len(X)!=len(Y)
        with self.assertRaises(ValueError):
            akima(np.array([0]), np.array([1, 2]), np.array([0, 1, 2]))

        # Test error: any(dt) <= 0
        with self.assertRaises(ValueError):
            akima(np.array([1, 0, 2]), np.array([1, 2]), np.array([0, 1, 2]))
        with self.assertRaises(ValueError):
            akima(np.array([0, 0, 2]), np.array([1, 2]), np.array([0, 1, 1]))

        # Test for correct responses
        T = np.array([0, 1, 2, 3, 4], dtype)
        S = np.array([0, 1, -1, -1, 5], dtype)
        t = np.array([i / 2.0 for i in range(9)], dtype)

        _t, s = emd.spline_points(t, np.array((T, S)))
        s_true = np.array([S[0], 0.9125, S[1], 0.066666, S[2], -1.35416667, S[3], 1.0625, S[4]], dtype)

        self.assertTrue(np.allclose(s_true, s), "Comparing akima with true")

        s_np = akima(np.array(T), np.array(S), np.array(t))
        self.assertTrue(np.allclose(s, s_np), "Shouldn't matter if with numpy")

    def test_cubic(self):
        dtype = np.float64

        emd = EMD()
        emd.spline_kind = "cubic"
        emd.DTYPE = dtype

        T = np.array([0, 1, 2, 3, 4], dtype=dtype)
        S = np.array([0, 1, -1, -1, 5], dtype=dtype)
        t = np.arange(9, dtype=dtype) / 2.0

        # TODO: Something weird with float32.
        # Seems to be SciPy problem.
        _t, s = emd.spline_points(t, np.array((T, S)))

        s_true = np.array([S[0], 1.203125, S[1], 0.046875, S[2], -1.515625, S[3], 1.015625, S[4]], dtype=dtype)
        self.assertTrue(np.allclose(s, s_true, atol=0.01), "Comparing cubic")

        T = T[:-2].copy()
        S = S[:-2].copy()
        t = np.arange(5, dtype=dtype) / 2.0

        _t, s3 = emd.spline_points(t, np.array((T, S)))

        s3_true = np.array([S[0], 0.78125, S[1], 0.28125, S[2]], dtype=dtype)
        self.assertTrue(np.allclose(s3, s3_true), "Compare cubic 3pts")

    def test_slinear(self):
        dtype = np.float64

        emd = EMD()
        emd.spline_kind = "slinear"
        emd.DTYPE = dtype

        T = np.array([0, 1, 2, 3, 4], dtype=dtype)
        S = np.array([0, 1, -1, -1, 5], dtype=dtype)
        t = np.arange(9, dtype=dtype) / 2.0

        _t, s = emd.spline_points(t, np.array((T, S)))

        s_true = np.array([S[0], 0.5, S[1], 0, S[2], -1, S[3], 2, S[4]], dtype=dtype)
        self.assertTrue(np.allclose(s, s_true), "Comparing SLinear")


if __name__ == "__main__":
    unittest.main()
